'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _utils = require('./utils');

var _enhanceReducer = require('./enhanceReducer');

var _enhanceReducer2 = _interopRequireDefault(_enhanceReducer);

var _AppStateWithEffects = require('./AppStateWithEffects');

var _AppStateWithEffects2 = _interopRequireDefault(_AppStateWithEffects);

/**
 * Wraps Store `getState` method. Instead of returning just plain old state,
 * it's assumed that the returned state is instance of `AppStateWithEffects`,
 * therefore it proxies the call to `getAppState`.
 *
 * @param {Object} Original Store
 * @returns {Function} Wrapped `getState`
 */
var wrapGetState = function wrapGetState(store) {
  return function () {
    var state = store.getState();

    if (state instanceof _AppStateWithEffects2['default']) {
      return state.getAppState();
    } else {
      return state;
    }
  };
};

exports.wrapGetState = wrapGetState;
/**
 * Wraps Store `dispatch` method. The original `dispatch` is called first, then
 * it iterates over all the effects returned from the `getState` function.
 * Every effect is then executed and dispatch is passed as the first argument.
 *
 * @param {Object} Original Store
 * @returns {Function} Wrapped `dispatch`
 */
var wrapDispatch = function wrapDispatch(store) {
  return function (action) {
    // Let's just dispatch original action,
    // the original dispatch might actually be
    // enhanced by some middlewares.
    var result = store.dispatch(action);

    var effects = store.getState().getEffects();

    (0, _utils.invariant)(effects.every(_utils.isFunction), 'It\'s allowed to yield only functions (side effect)');

    // Effects are executed after action is dispatched
    effects.forEach(function (effect) {
      return effect(wrapDispatch(store));
    });

    return result;
  };
};

exports.wrapDispatch = wrapDispatch;
/**
 * Creates enhanced store factory, which takes original `createStore` as argument.
 * The store's `dispatch` and `getState` methods are wrapped with custom implementation.
 *
 * wrappedDispatch calls the original dispatch and executes all the side effects.
 *
 * wrappedGetState unwraps original applicaiton state from `AppStateWithEffects`
 *
 * @param {Function} Original createStore implementation to be enhanced
 * @returns {Function} Store factory
 */

exports['default'] = function (createStore) {
  return function (rootReducer, initialState) {
    var store = createStore((0, _enhanceReducer2['default'])(rootReducer), new _AppStateWithEffects2['default'](initialState, []));

    return _extends({}, store, {
      dispatch: wrapDispatch(store),
      getState: wrapGetState(store)
    });
  };
};