'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _AppStateWithEffects = require('./AppStateWithEffects');

var _AppStateWithEffects2 = _interopRequireDefault(_AppStateWithEffects);

var _utils = require('./utils');

/**
 * AppStateOrEffect mapper - maps Iterable to AppStateOrEffect data structure.
 * The last element in Iterable chain is Application State the rest are some side effect.
 *
 * @param {any} Side effect Function or Application State any
 *
 * @param {Boolean} Last element in iterable
 * @returns {Object} AppStateOrEffect - Object containing isEffect(Boolean) and value(Function|Any)
 */
var mapIterableToEffectsAndAppState = function mapIterableToEffectsAndAppState(value, last) {
  return {
    isEffect: !last,
    value: value
  };
};

/**
 * AppStateOrEffect mapper - extracts just the value.
 *
 * @param {Object} Either Application State or Effect
 * @returns {any} Might be either Function (for Effect) or any for Application State
 */
var mapValue = function mapValue(appStateOrEffect) {
  return appStateOrEffect.value;
};

/**
 * Predicate function - decides whether appStateOrEffect is an Effect.
 *
 * @param {Object} Either Application State or Effect
 * @returns {Boolean}
 */
var effectsPredicate = function effectsPredicate(appStateOrEffect) {
  return appStateOrEffect.isEffect;
};

/**
 * Predicate function - decides whether appStateOrEffect is an Application State.
 *
 * @param {Object} Either Application State or Effect
 * @returns {Boolean}
 */
var appStatePredicate = function appStatePredicate(appStateOrEffect) {
  return !appStateOrEffect.isEffect;
};

/**
 * Reducer enhancer. Iterates over generator like reducer reduction and accumulates
 * all the side effects and reduced application state.
 *
 * @param {Function} Root reducer in form of generator function
 * @returns {Function} Reducer which returns AppStateWithEffects instead of simple Application state
 */

exports['default'] = function (rootReducer) {
  return function (stateWithEffects, action) {
    if (stateWithEffects === undefined) stateWithEffects = new _AppStateWithEffects2['default']();

    (0, _utils.invariant)((0, _utils.isFunction)(rootReducer), 'Provided root reducer is not a function.');

    // Calling the Root reducer returns an Iterable
    var iterable = rootReducer(stateWithEffects.getAppState(), action);

    (0, _utils.invariant)((0, _utils.isIterable)(iterable), 'Root reducer must be a generator function and therefore return Iterable');

    // Iterable returned by Root reducer is mapped into array of values.
    // Last value in the array is reduced application state all the other values
    // are just side effects.
    var appStateAndEffects = (0, _utils.mapIterable)(iterable, mapIterableToEffectsAndAppState);

    // It's necessary to separate array of effects and Application state from flat array
    var effects = appStateAndEffects.filter(effectsPredicate).map(mapValue);
    var appState = appStateAndEffects.filter(appStatePredicate).map(mapValue);

    (0, _utils.invariant)(!(0, _utils.isUndefined)((0, _utils.first)(appState)), 'Root reducer does not return new application state. Undefined is returned');

    return new _AppStateWithEffects2['default']((0, _utils.first)(appState), effects);
  };
};

module.exports = exports['default'];