/**
 * Simple invariant check.
 *
 * @param {Boolean} A condition to be met
 *
 * @param {String} An exception message to be thrown
 * @returns {void}
 */
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
var invariant = function invariant(condition, message) {
  if (!condition) {
    throw new Error('Invariant violation: ' + message);
  }
};

exports.invariant = invariant;
/**
 * Checks whether provided argument is a function.
 *
 * @param {any} Anything
 *
 * @returns {Boolean} Result of function check
 */
var isFunction = function isFunction(any) {
  return typeof any === 'function';
};

exports.isFunction = isFunction;
/**
 * Checks whether provided argument is undefined.
 *
 * @param {any} Anything
 *
 * @returns {Boolean} Result of undefined check
 */
var isUndefined = function isUndefined(any) {
  return typeof any === 'undefined';
};

exports.isUndefined = isUndefined;
/**
 * Checks whether provided argument is iterable. The value must be defined and
 * must contain function named next.
 *
 * @param {any} Anything
 *
 * @returns {Boolean} Result of iterable check
 */
var isIterable = function isIterable(value) {
  return !isUndefined(value) && isFunction(value.next);
};

exports.isIterable = isIterable;
/**
 * Map implementation which takes iterable as an argument.
 *
 * @param {Iterable}
 *
 * @param {Function}
 * @returns {Array} Array mapped by mapper function
 */
var mapIterable = function mapIterable(iterable, mapper) {
  invariant(isIterable(iterable), 'First argument passed to mapIterable must be iterable');

  invariant(isFunction(mapper), 'Second argument passed to mapIterable must be a function');

  // Clojure like recur loop
  // It's not ideal to use for..of as it does not
  // return the last value in iteration loop
  var recur = function recur(_x) {
    var _again = true;

    _function: while (_again) {
      var acc = _x;
      _again = false;

      var next = iterable.next();
      acc.push(mapper(next.value, next.done));

      // ES6 tail call
      if (next.done) {
        return acc;
      } else {
        _x = acc;
        _again = true;
        next = undefined;
        continue _function;
      }
    }
  };

  return recur([]);
};

exports.mapIterable = mapIterable;
/**
 * Returns first element in a non-empty array;
 *
 * @param {Array}
 * @returns {any} First element in the provided Array
 */
var first = function first(arr) {
  invariant(Array.isArray(arr), 'Provided argument is not array');

  invariant(arr.length > 0, 'Provided array is empty');

  return arr[0];
};
exports.first = first;